/*
 * Copyright (c) 2017, Red Hat Inc. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package org.openjdk.gcbench.tests;

import org.openjdk.gcbench.HeapSizeManager;

public class Dimensions {

    public static Dimension heapSize(int steps) {
        int maxHeap = HeapSizeManager.MAX_HEAP;
        return new Dimension(DimensionType.HEAPSIZE, Sequence.steps(maxHeap / 16, maxHeap - maxHeap / 16, steps));
    }

    public static Dimension heapSize(Sequence seq) {
        return new Dimension(DimensionType.HEAPSIZE, seq);
    }

    public static Dimension lds(int steps) {
        int maxHeap = HeapSizeManager.MAX_HEAP;
        return new Dimension(DimensionType.LDS,
                Sequence.steps(maxHeap / 16, maxHeap - maxHeap / 16, steps));
    }

    public static Dimension size(Sequence seq) {
        return new Dimension(DimensionType.SIZE, seq);
    }

    public static Dimension threads(Sequence seq) {
        return new Dimension(DimensionType.THREADS, seq);
    }

    public static Dimension jvmMode() {
        return new Dimension(DimensionType.JVM_MODE, Sequence.jvmModes());
    }

    public static Dimension javaType() {
        return new Dimension(DimensionType.JAVA_TYPE, Sequence.javaType());
    }
}
